@@ -21,6 +21,9 @@ use std::path::Path;
 use std::fs::File;
 use std::str;
 
+use base64::Engine;
+use base64::engine::general_purpose::STANDARD as base64std;
+
 use sequoia_openpgp as openpgp;
 use openpgp::armor;
 use openpgp::Error;
@@ -203,7 +206,7 @@ impl AutocryptHeader {
 
         let mut buf = Vec::new();
         self.key.as_ref().unwrap().serialize(&mut buf)?;
-        write!(o, "keydata={} ", base64::encode(&buf))?;
+        write!(o, "keydata={} ", base64std.encode(&buf))?;
         Ok(())
     }
 
@@ -267,8 +270,19 @@ impl AutocryptHeaders {
 
             if let Some(rest) = line.strip_prefix(FROM) {
                 headers.from = Some(rest.trim_matches(' ').into());
-            } else if line.starts_with(AUTOCRYPT) || line.starts_with(AUTOCRYPT_GOSSIP) {
-                headers.headers.push(Self::decode_autocrypt_like_header(&line));
+            } else {
+                if let Some((key, value)) =
+                    if let Some(v) = line.strip_prefix(AUTOCRYPT) {
+                        Some((AutocryptHeaderType::Sender, v))
+                    } else if let Some(v) = line.strip_prefix(AUTOCRYPT_GOSSIP) {
+                        Some((AutocryptHeaderType::Gossip, v))
+                    } else {
+                        None
+                    }
+                {
+                    headers.headers.push(
+                        Self::decode_autocrypt_like_header(key, value));
+                }
             }
         }
 
@@ -278,49 +292,42 @@ impl AutocryptHeaders {
     /// Decode header that has the same format as the Autocrypt header.
     /// This function should be called only on "Autocrypt" or "Autocrypt-Gossip"
     /// headers.
-    fn decode_autocrypt_like_header(line: &str) -> AutocryptHeader {
-        let mut parts = line.splitn(2, ": ");
-        let header_name = parts.next().unwrap();
-        let ac_value = parts.next().unwrap();
-
-        let header_type = match header_name {
-            "Autocrypt" => AutocryptHeaderType::Sender,
-            "Autocrypt-Gossip" => AutocryptHeaderType::Gossip,
-            other => panic!("Expected Autocrypt header but found: {}", other)
-        };
-
+    fn decode_autocrypt_like_header(header_type: AutocryptHeaderType,
+                                    ac_value: &str)
+        -> AutocryptHeader
+    {
         let mut header = AutocryptHeader::empty(header_type);
 
         for pair in ac_value.split(';') {
             let pair = pair
-                .splitn(2, |c| c == '=')
+                .splitn(2, '=')
                 .collect::<Vec<&str>>();
 
-            let (key, value) : (String, String) = if pair.len() == 1 {
+            let (key, value) : (&str, String) = if pair.len() == 1 {
                 // No value...
-                (pair[0].trim_matches(' ').into(), "".into())
+                (pair[0].trim_matches(' '), "".into())
             } else {
-                (pair[0].trim_matches(' ').into(),
+                (pair[0].trim_matches(' '),
                  pair[1].trim_matches(' ').into())
             };
 
             if key == "keydata" {
-                if let Ok(decoded) = base64::decode(
-                    &value.replace(" ", "")[..]) {
+                if let Ok(decoded) = base64std.decode(
+                    &value.replace(' ', "")[..]) {
                     if let Ok(cert) = Cert::from_bytes(&decoded[..]) {
                         header.key = Some(cert);
                     }
                 }
             }
 
-            let critical = !key.is_empty() && &key[0..1] == "_";
+            let (critical, key) = if let Some(key) = key.strip_prefix('_') {
+                (true, key)
+            } else {
+                (false, key)
+            };
             header.attributes.push(Attribute {
                 critical,
-                key: if critical {
-                    key[1..].to_string()
-                } else {
-                    key
-                },
+                key: key.to_string(),
                 value,
             });
         }
@@ -555,7 +562,7 @@ impl AutocryptSetupMessage {
         -> Result<AutocryptSetupMessageParser<'a>> {
         // The outer message uses ASCII-armor.  It includes a password
         // hint.  Hence, we need to parse it aggressively.
-        let mut r = armor::Reader::new(
+        let mut r = armor::Reader::from_reader(
             r, armor::ReaderMode::Tolerant(Some(armor::Kind::Message)));
 
         // Note, it is essential that we call r.headers here so that
@@ -683,7 +690,7 @@ impl<'a> AutocryptSetupMessageParser<'a> {
     /// `AutocryptSetupMessageParser::parse()` to extract the
     /// `AutocryptSetupMessage`.
     pub fn decrypt(&mut self, passcode: &Password) -> Result<()> {
-        if ! self.pp.encrypted() {
+        if self.pp.processed() {
             return Err(
                 Error::InvalidOperation("Already decrypted".into()).into());
         }
@@ -707,7 +714,7 @@ impl<'a> AutocryptSetupMessageParser<'a> {
     /// If the payload is malformed, returns
     /// `Error::MalformedMessage`.
     pub fn parse(self) -> Result<AutocryptSetupMessage> {
-        if self.pp.encrypted() {
+        if ! self.pp.processed() {
             return Err(
                 Error::InvalidOperation("Not decrypted".into()).into());
         }
@@ -735,7 +742,7 @@ impl<'a> AutocryptSetupMessageParser<'a> {
             // The inner message consists of an ASCII-armored encoded
             // Cert.
             let (prefer_encrypt, cert) = {
-                let mut r = armor::Reader::new(
+                let mut r = armor::Reader::from_reader(
                     &mut pp,
                     armor::ReaderMode::Tolerant(
                         Some(armor::Kind::SecretKey)));
@@ -1126,4 +1133,20 @@ mod test {
         ];
         let _ = AutocryptHeaders::from_bytes(&data);
     }
+
+    #[test]
+    fn issue_1012() {
+        let data: Vec<u8> = vec![
+            0x41, 0x75, 0x74, 0x6f, 0x63, 0x72, 0x79, 0x70, 0x74, 0x2d, 0x47, 0x6f, 0x73, 0x73, 0x69,
+            0x70, 0x3a, 0x20, 0xc8, 0x84, 0x01, 0x42, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x25, 0x25, 0x25,
+            0x25, 0x42, 0x25, 0x3f, 0x21, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25,
+            0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x00, 0x40, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25,
+            0x25, 0x22, 0x6b, 0x25, 0x25, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x25, 0x25, 0x25, 0x25, 0x25,
+            0x25, 0x25, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x25, 0x25, 0x25, 0x25, 0x25,
+            0x25, 0x25, 0x25, 0x25, 0x25, 0x25, 0x25,
+        ];
+
+        AutocryptHeaders::from_bytes(&data).expect("parses");
+    }
 }
